package com.osi.notification;

import java.io.Serializable;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Session;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.transaction.UserTransaction;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class NotificationHelper {
	
	private static final Logger log = LoggerFactory.getLogger(NotificationHelper.class);
	private static ConnectionFactory factory;
	private static Context ctx;
	private static UserTransaction tx;

	static {
		try {
			ctx = new InitialContext();
			factory = (ConnectionFactory)ctx.lookup("java:/JmsXA");
			tx = (UserTransaction)ctx.lookup("UserTransaction");
		} catch (Exception e) {
			log.error("Failed to get necessary resources.", e);
		}
	}

	public static void sendNotificationWithTx(Object notification, Destination destination) throws JMSException {
		log.trace("Sending Notification With Tx");
		try {
			tx.begin();
			sendNotification(notification, destination);
			tx.commit();
		} catch (JMSException e) {
			try { tx.rollback(); }
			catch (Exception e1) {}
			throw e;
		} catch (Exception e) {
			try { tx.rollback(); }
			catch (Exception e1) {}
			log.error("Transaction Error sending notification", e);
		}
	}
	
	public static void sendNotification(Object notification, Destination destination) throws JMSException {
		log.debug("NOTIFICATION: {}", notification);
        Connection connection = null;
        Session session = null;
        MessageProducer producer = null;
        try {
            connection = factory.createConnection();
            session = connection.createSession(true, Session.AUTO_ACKNOWLEDGE);
            log.trace("THE DESTINATION: {}", destination);
            producer = session.createProducer(destination);
            
            ObjectMessage message = session.createObjectMessage();
            message.setObject((Serializable)notification);
            message.setStringProperty("myproperty", "myvalue");
            log.trace("SENDING JMS MESSAGE: {} WITH PROPERTIES: {}", message, message.getPropertyNames());
            
            producer.send(message);
        }
        finally {
        	if (producer != null) {
        		try { producer.close(); }
        		catch (Exception e) {}
        	}
			if (session != null) {
				try { session.close(); }
				catch (Exception e) {}
			}
			if (connection != null) {
				try { connection.close(); }
				catch (Exception e) {}
			}
        }
	}

	
}
